/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gameLauncher.h"

 
GameLauncher::GameLauncher(newtGame game, Mesh* launcherMesh, GraphicOptions *graphics): EntityWithPosition() {
	mNumClick = game;
	
	setMesh(launcherMesh);
	Material *material;
	
	switch(game) {
		case NP_TIC_TAC_TOE:
			mNameGame = "Tic-Tac-Toe";
			material = new Material ("ticTacToeLauncherMaterial", 5);
			material->setEmissionColor(-0.98);
			material->addTexture("GamesNames/TicTacToe.png", true, 0, COLOR_MAP, GL_REPEAT, 3., 3., true, graphics);
			material->addTexture("GamesNames/TicTacToeGloss.png", true, 0, GLOSS_MAP, GL_REPEAT, 3., 3., true, graphics);
			material->addTexture("GamesNames/TicTacToeEmission.png", true, 0, EMISSION_MAP, GL_REPEAT, 3., 3., true, graphics);								
			break;
			
		case NP_SCORE_FOUR:
			mNameGame = "Score Four";		
			material = new Material ("scoreFourLauncherMaterial", 5);
			material->setEmissionColor(-0.98);			
			material->addTexture("GamesNames/ScoreFour.png", true, 0, COLOR_MAP, GL_REPEAT, 3., 3., true, graphics);
			material->addTexture("GamesNames/ScoreFourGloss.png", true, 0, GLOSS_MAP, GL_REPEAT, 3., 3., true, graphics);	
			material->addTexture("GamesNames/ScoreFourEmission.png", true, 0, EMISSION_MAP, GL_REPEAT, 3., 3., true, graphics);					
			break;
			
		case NP_KEPLER_BUBBLE:
			mNameGame = "Kepler Bubble";		
			material = new Material ("keplerBubbleMaterial", 1);
			material->setEmissionColor(-0.98);			
			material->addTexture("GamesNames/KeplerBubble.png", true, 0, COLOR_MAP, GL_REPEAT, 3., 3., true, graphics);
			material->addTexture("GamesNames/KeplerBubbleGloss.png", true, 0, GLOSS_MAP, GL_REPEAT, 3., 3., true, graphics);	
			material->addTexture("GamesNames/KeplerBubbleEmission.png", true, 0, EMISSION_MAP, GL_REPEAT, 3., 3., true, graphics);			
			break;
		
	}
	
	setMaterial (material);
	
}
	
GameLauncher::~GameLauncher() {

}
	

std::string GameLauncher::getName() {
	return mNameGame;
}


