/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "frameBufferObject.h"

FrameBufferObject::FrameBufferObject(int width, int height): mWidth(width), mHeight(height) {
 
	glGenFramebuffersEXT(1, &idFrame);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, idFrame);

	glGenRenderbuffersEXT(1, &idRenderDepth);
	glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, idRenderDepth);
	glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, width, height);
	
	//glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, 2, GL_DEPTH_COMPONENT, width, height);
	// do not compile :-(

	glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, idRenderDepth);

	tex = new TextureLayer(width, height);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,	GL_TEXTURE_2D, tex->getId(), 0);

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	
	GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	if(status == GL_FRAMEBUFFER_COMPLETE_EXT) {
		std::cout << "Framebuffer object is complete." << std::endl;
	}
}

FrameBufferObject::~FrameBufferObject() {
	glDeleteFramebuffersEXT(1, &idFrame);
	glDeleteRenderbuffersEXT(1, &idRenderDepth);
	delete tex;
}

 void FrameBufferObject::bind(){
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, idFrame);
	glPushAttrib(GL_VIEWPORT_BIT);	
	glViewport(0, 0, mWidth, mHeight); 
}

void FrameBufferObject::unBind(){
	glPopAttrib();	
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
}


TextureLayer* FrameBufferObject::getTexture() {
	return tex;
}
