/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MESH_H_
#define _MESH_H_

#include "glUtil.h"
#include "vector3.h"
#include "color.h"
#include "shaderManager.h"


#define BUFFER_OFFSET(a) ((char*)NULL + (a))

enum bufferName{VERTEX_ID=0, COLOR_ID=1, NORMAL_ID=2, TEX_COORD_ID=3, FOG_COORD_ID=4, TANGENT_ID=5};


struct VertexArrayData {
	Vector3 *mpVertices;
	Color *mpColors;
	Vector3 *mpNormals;
	GLfloat *mpTexCoord;
	GLfloat *mpFogCoord;
	Vector3 *mpTangents;
		
	GLuint **mpIndices;	
};

struct Geometry {
	int nbVertices;	
	int nbIndexArrays;
	GLenum *primType;
	int *nbIndices;
};
	
	
class Mesh
{
public:
	Mesh(std::string name, bool colorArray, bool normalArray, bool fogCoordArray, bool tangentArray);
	virtual ~Mesh();
	
	void setName(std::string name);
	std::string getName();
		
	int getNbEntities();
		
	void deleteVertexArrayData();
				
	virtual void createVertices(Vector3 *ptr) = 0;
	virtual void createColors(Color *ptr) = 0;
	virtual void createNormals(Vector3 *ptr) = 0;
	virtual void createTexCoords(GLfloat *ptr) = 0;
	virtual void createFogCoords(GLfloat *ptr) = 0;
	virtual void createTangents(Vector3 *ptr) = 0;			
	
	virtual void createIndices(int numArray, GLuint *ptr) = 0;		

	virtual void initFirstParams() = 0;
				
	void initialize(int nbTextures);
	bool isInitialized();		
		
	void draw(int nbTextures);
		
	void incNbEntities();
	void decNbEntities();
		
protected:
	std::string mName;
	
	void buildVBOs(bool texCoordArray);
	void initVertexArray(bool texCoordArray);
		
	void activateClientStates();
		
	void drawWithVBO(int nbTextures);
	void drawWithVertexArrays(int nbTextures);		
		
	Vector3 *mpUniformNormal;
	Vector3 *mpUniformTangent;
		
	GLuint vertexIds[6];
	GLuint *primIds;
		
	struct VertexArrayData *data;
	struct Geometry geometry;
		
	bool mIsInitialized;
	int mNbEntities;	//number of Entity which uses this mesh
};




#endif // _MESH_H_
