/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "shaderManager.h"

ShaderManager* shaderMana = 0;

ShaderManager::ShaderManager(): currentShaderType(FIXED_PIPELINE), currentShaderId(0), currentMaterialShader(NULL), sky(0)
{

}

ShaderManager::~ShaderManager() {

	
}


shaderType ShaderManager::getCurrentShaderType() {
	return currentShaderType;
}


int ShaderManager::getCurrentLightKey() {
	return currentLightKey;
}

int ShaderManager::getCurrentMaterialKey() {
	return currentMaterialKey;
}


void ShaderManager::setLightKey(int key) {
	currentLightKey = key;
}

void ShaderManager::setMaterialKey(int key) {
	currentMaterialKey = key;
}


void ShaderManager::setAmbientColors(Color groundColor, Color skyColor, Vector3 hemiAxis) {
	//if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setAmbientColors (groundColor, skyColor, hemiAxis);
	//}
}

void ShaderManager::setNumMaps(int color, int normal, int gloss, int emi) {
	//if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setNumMaps (color, normal, gloss, emi);
	//}
}


void ShaderManager::setIdColorMap (int id) {
	if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setIdColorMap (id);
	}
}

void ShaderManager::setIdNormalMap(int id) {
	if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setIdNormalMap (id);
	}
}

void ShaderManager::setIdGlossMap(int id) {
	if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setIdGlossMap (id);
	}
}

void ShaderManager::setIdEmiMap(int id) {
	if((currentShaderType == PER_FRAG_LIGHT)&&(currentMaterialShader != NULL)) {
		currentMaterialShader->setIdEmiMap (id);
	}
}


int ShaderManager::getLocationTangent() {
	if(currentMaterialShader != NULL) {
		return currentMaterialShader->getLocationTangent();
	} else {
		return 0;
	}
}



void ShaderManager::activateFixedPipeline() {
	glUseProgram(0);
	currentShaderType = FIXED_PIPELINE;		
}


bool ShaderManager::compilePerFragLight(int minDirLights, int maxDirLights,
							 int minSpotLights, int maxSpotLights,
							 int minPointLights, int maxPointLights,
							 std::set<int> matKeys) {

	int amb, emi, gloss, normal, color;
	int dir, spot, point;												   
													   
	int materialKey;
	int lightKey;
															   
	for(dir=minDirLights;dir<=maxDirLights;dir++) {
		lightKey = dir*64;
		for(spot=minSpotLights;spot<=maxSpotLights;spot++) {
			lightKey += spot*8;
			for(point=minPointLights;point<=maxPointLights;point++) {
				lightKey+=point;
				
				for(std::set<int>::iterator it = matKeys.begin(); it!=matKeys.end(); it++){
					materialKey = *it;
					
					int r;
					emi = materialKey /(4*4*4);
					r = materialKey % (4*4*4);
					
					gloss = r / (4*4);
					r = r % (4*4);
					
					normal = r / 4;
					
					color = r % 4;

					std::pair<int, int> keys = std::make_pair(lightKey,materialKey);
										
					if(perFragLightPrograms.find(keys) == perFragLightPrograms.end()) {
						PerFragLightShader *shader = new PerFragLightShader(dir, spot, point, color, normal, gloss, emi);
						if(shader->getId() == 0) {
							return false;
						} else {
							perFragLightPrograms[keys] = shader;
						}
					}		
				}
			lightKey-=maxPointLights;
			}
		lightKey-=maxSpotLights*8;
		}
	lightKey-=maxDirLights*64;
	}
	
	return true;	
								 
}


void ShaderManager::activateMaterialShader(int lightKey, int materialKey, shaderType type) {
	
	GLuint shaderId = perFragLightPrograms[std::make_pair(lightKey,materialKey)]->getId();
	
	glUseProgram(shaderId);
	currentLightKey = lightKey;	
	currentMaterialKey = materialKey;
	
	currentMaterialShader = perFragLightPrograms[std::make_pair(lightKey,materialKey)];

	currentShaderType = type;
	currentShaderId = shaderId;	
}



bool ShaderManager::compileSky() {
	sky = new SkyShader();
	
	return sky->getId();
}

void ShaderManager::activateSky() {
	currentShaderType = SKY;
	GLuint shaderId = sky->getId();
	glUseProgram(shaderId);
	currentShaderId = shaderId;	
}


SkyShader* ShaderManager::getSkyShader() {
	return sky;
}
