/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "waterPlane.h"


WaterPlane::WaterPlane(Color color, float height, Mesh *waterSurface, Vector3 center): 
	mColor(color), mHeight(height), mpWaterSurface(waterSurface), mCenter(center), infinite(false) {
	mpWaterSurface->initialize(0);
}

WaterPlane::WaterPlane(Color color, float height, float farClip):
	mColor(color), mHeight(height), infinite(true), mFarClip(farClip) {
	mpWaterSurface = new RectangleMesh(farClip, 2, 0., farClip, 2, 0.);
	mpWaterSurface->initialize(0);
}

WaterPlane::~WaterPlane() {
}

Color WaterPlane::getColor() {
	return mColor;
}

float WaterPlane::getHeight () {
	return mHeight;
}

void WaterPlane::drawSurface() {
	glPushMatrix();
	glEnable (GL_COLOR_MATERIAL);
	glColor4f(mColor.r,mColor.g,mColor.b,0.2);
	glTranslatef(mCenter.x, mCenter.y, mCenter.z);
	glRotatef(-90.,1.,0.,0.);
	mpWaterSurface->draw(0);
	glDisable (GL_COLOR_MATERIAL);	
	glPopMatrix();
}




