/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _WINDOW_H_
#define _WINDOW_H_

  
#include <SDL/SDL.h>
#include <set>
#include <iostream>
#include <sstream>

#include "./newtEngine/glUtil.h"
#include "game.h"
#include "newtPuzzle.h"
#include "./gameChooser/gameChooser.h"
#include "menu.h"
#include "./newtEngine/graphicOptions.h"
#include "keyboardOptions.h"
#include "./newtEngine/screenshot.h"

using namespace std;

/* print useful information about the SDL OpenGL rendering context
 */
void window_printSDLGLinfos();

void* window_loadGLextension(const char *fname);
  
/* initialize the OpenGL rendering context and create a window
 */
bool window_init( int *argc, char **argv, char *window_title, char *icon_title);
 

int getTime();

void window_quit();

/* exit the game and save configuration
 */
void window_exit();
 
void window_setAntiAliasing(int nbSample);

/* keyboard
 */
void window_enable_key_repeat(bool enabled); 
// return true if the key is repeatable
bool keyboardEvent(SDLKey key, double timeInterval) ;


/* mouse
 */
void window_warpPointer(int x, int y);
void window_showCursor(bool show); 
void window_captureCursor(bool captureCursor);
  

/* Processes events. Never returns.
 */
void window_event(bool isActive);


#endif // _WINDOW_H_
